PTools Hook
===========

The PTools Hook module allows to define hook implementations as class methods to
help writing fully object-oriented Drupal code.

Since hooks only work with functions, this relies on two alternative approaches
under the hood:

* Static code generation via the `drush ptools-hook-dump-implementations`
  command, which will create a `ptools.hook.php` file in the webroot.
* Runtime inclusion of dynamically-generated code via `eval`.

Ideally one would rely on the `eval` workflow while developing locally and, once
done, would dump the hook implementations via the `drush` command. Two variables
are available to disable `eval` and `include` respectively in production and
local environments:

* `ptools_hook_allow_runtime_implementations`
* `ptools_hook_include_implementations`

Hook implementations are loaded on module load, so bootstrap hooks are not
supported.


Hook implementation
-------------------
Classes providing hook implementations should implement
`Drupal\ptools_hook\HookListenerInterface` and be defined in the
`Drupal\my_module\Hook` namespace, where `my_module` is the name of the provider
module. Method names should follow the `onHookName` convention to be discovered.

Here is an example:

```php
<?php

// Only classes in the "Hook" sub-namespace will be discovered.
namespace Drupal\my_module\Hook;

use Drupal\ptools\SingletonTrait;
use Drupal\ptools_hook\HookListenerInterface;

// Only classes implementing "\Drupal\ptools_hook\HookListenerInterface" will be
// discovered. It is also possible to implement the singleton pattern via
// "\Drupal\ptools\SingletonTrait", so the singleton instance will be used
// instead of instantiating a new object on each hook invocation.
class TestHookListener implements HookListenerInterface {

  use SingletonTrait;

  /**
   * Implements hook_page_alter().
   */
  public function onPageAlter(array &$page) {
    // ...
  }

}
```

A module may define multiple hook listeners providing implementations for the
same hook: in this case both methods will be invoked. The invocation order will
be determined by comparing the parent class names alphabetically:
`\Drupal\my_module\Hook\ListenerA::onPageAlter()` will be invoked before
`\Drupal\my_module\Hook\ListenerB::onPageAlter()`. However, when using multiple
listeners, there is a key limitation: the only supported return values are
arrays, any scalar value will be treated as `NULL`.
