<?php

/**
 * @file
 * Drush commands for "ptools_hook".
 */

use Drupal\ptools_hook\HookImplementationsHandler;

/**
 * Implements hook_drush_command().
 */
function ptools_hook_drush_command() {
  $items = [];

  $items['ptools-hook-dump-implementations'] = [
    'description' => 'Dumps the available hook implementations to the specified file.',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
    'aliases' => ['pt-dh'],
    'options' => [
      'skip-autoload-rebuild' => 'Skips autoload rebuild. Defaults to FALSE.',
    ],
  ];

  return $items;
}

/**
 * Dumps the available hook implementations to the specified file.
 *
 * @param string|null $file_path
 *   The path of the target file. Defaults to the final name/position in the
 *   Drupal root.
 */
function drush_ptools_hook_dump_implementations($file_path = NULL) {
  if (!isset($file_path)) {
    $file_path = DRUPAL_ROOT . '/ptools.hook.php';
  }

  if (!drush_get_option('skip-autoload-rebuild', FALSE)) {
    autoload(TRUE);
  }

  $implementations_handler = HookImplementationsHandler::get();
  $info = $implementations_handler->getListenersInfo();

  // Sort listener info alphabetically by hook and module.
  foreach ($info as $module => $module_info) {
    ksort($module_info);
  }
  ksort($info);

  $hook_implementations = $implementations_handler->dumpImplementations($info);

  // Add a check that "autoload()" exists to make sure that the system is
  // properly bootstrapped.
  $php_code = "<?php

/**
 * @file
 * PTools Hook auto-generated hook implementations file.
 *
 * @noinspection PhpMethodParametersCountMismatchInspection
 * @noinspection PhpVoidFunctionResultUsedInspection
 *
 * @see drush_ptools_hook_dump_implementations()
 */

if (!function_exists('autoload')) {
  return FALSE;
}
{$hook_implementations}
return TRUE;
";

  file_put_contents($file_path, $php_code);
}
