<?php

namespace Drupal\ptools;

/**
 * Provides a context stack.
 */
class CurrentContext implements \ArrayAccess {

  use SingletonTrait;

  /**
   * The context stack.
   *
   * @var \SplStack
   */
  protected $data;

  /**
   * CurrentStateRepository constructor.
   */
  public function __construct() {
    $this->data = new \SplStack();
  }

  /**
   * {@inheritdoc}
   */
  public static function create() {
    $current_state = new static();
    $current_state->pushContext();
    return $current_state;
  }

  /**
   * Pushes a new blank context on top of the stack.
   */
  public function pushContext(): void {
    $state = new \stdClass();
    $this->data->push($state);
  }

  /**
   * Pops the top context off of the stack.
   */
  public function popContext(): void {
    $this->data->pop();
  }

  /**
   * Returns the current context.
   *
   * @return \stdClass|null
   *   The current context or NULL if none is available.
   */
  protected function getCurrentContext(): ?\stdClass {
    $state = $this->data->top() ?: NULL;
    return $state;
  }

  /**
   * {@inheritdoc}
   */
  public function offsetExists($offset) {
    $state = $this->getCurrentContext();
    return isset($state->{$offset});
  }

  /**
   * {@inheritdoc}
   */
  public function offsetGet($offset) {
    $state = $this->getCurrentContext();
    return $state->{$offset} ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function offsetSet($offset, $value) {
    $state = $this->getCurrentContext();
    $state->{$offset} = $value;
  }

  /**
   * {@inheritdoc}
   */
  public function offsetUnset($offset) {
    $state = $this->getCurrentContext();
    unset($state->{$offset});
  }

}
