<?php

namespace Drupal\ptools\Entity;

/**
 * Base class for bundle-specific entity wrappers.
 */
abstract class EntityBundleWrapperBase extends EntityWrapper {

  /**
   * EntityBundleWrapper constructor.
   *
   * @param object|int $entity
   *   The entity object or its ID.
   */
  public function __construct($entity) {
    parent::__construct(static::getEntityType(), $entity);
  }

  /**
   * {@inheritdoc}
   */
  public static function wrapValues(array $values) {
    $entity_type = static::getEntityType();
    $bundle = static::getBundleName();
    $entity_info = entity_get_info($entity_type);

    $bundle_key = $entity_info['entity keys']['bundle'];
    if (!isset($values[$bundle_key])) {
      $values[$bundle_key] = $bundle;
    }

    $instances = field_info_instances($entity_type, $bundle);
    foreach ($instances as $field_name => $instance) {
      $value = $values[$field_name] ?? NULL;
      // @todo Add multilingual support.
      if (isset($value) && (!is_array($value) || !isset($value[LANGUAGE_NONE]))) {
        $field = field_info_field($field_name);
        $property_name = key($field['columns']);
        $item_values = is_scalar($value) ? [$value] : $value;
        $values[$field_name] = [];
        foreach ($item_values as $item_value) {
          $values[$field_name][LANGUAGE_NONE][] = [$property_name => $item_value];
        }
      }
    }

    return parent::wrapValues($values);
  }

  /**
   * Returns the entity type.
   *
   * @return string
   *  The entity type name.
   */
  protected static abstract function getEntityType(): string;

  /**
   * Returns the entity bundle.
   *
   * @return string
   *  The bundle name.
   */
  protected static abstract function getBundleName(): string;

}
