<?php

namespace Drupal\ptools\Render;

/**
 * A renderable object useful to defer rendering as much as possible.
 */
class Renderable implements \ArrayAccess {

  /**
   * A renderable array.
   *
   * @var array
   */
  protected $element;

  /**
   * Renderable constructor.
   *
   * @param array $element
   *   (optional) A renderable array.
   */
  public function __construct(array $element = []) {
    $this->element = $element;
  }

  /**
   * {@inheritdoc}
   */
  public function offsetExists($offset) {
    return array_key_exists($offset, $this->element);
  }

  /**
   * {@inheritdoc}
   */
  public function offsetGet($offset) {
    return $this->element[$offset];
  }

  /**
   * {@inheritdoc}
   */
  public function offsetSet($offset, $value) {
    $this->element[$offset] = $value;
  }

  /**
   * {@inheritdoc}
   */
  public function offsetUnset($offset) {
    unset($this->element[$offset]);
  }

  /**
   * Renders the element.
   *
   * @return string
   *   The rendered markup.
   */
  public function render() {
    if (!isset($this->element['#children'])) {
      drupal_render($this->element);
    }
    return $this->element['#children'];
  }

  /**
   * {@inheritdoc}
   */
  public function __toString() {
    return $this->render();
  }

}
