<?php

namespace Drupal\ptools\Utils;

use Drupal\ptools\DatabaseConnectionTrait;
use Drupal\ptools\SingletonTrait;

/**
 * Extension-related helper methods.
 */
class Extension {

  use SingletonTrait;
  use DatabaseConnectionTrait;

  /**
   * DatabaseRepositoryBase constructor.
   *
   * @param \DatabaseConnection $database
   *   The database connection.
   */
  public function __construct(\DatabaseConnection $database) {
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create() {
    return new static(
      \Database::getConnection()
    );
  }

  /**
   * Uninstalls a module missing from the file system.
   *
   * @param string $module
   *   The module name.
   * @param array $tables
   *   (optional) An array of table names provided by the missing module.
   *   Defaults to none.
   * @param array $variable_patterns
   *   (optional) An array of SQL string patterns identifying the variables to
   *   be deleted. Defaults to "MODULE_NAME_%".
   */
  public function uninstallMissingModule($module, array $tables = [], array $variable_patterns = NULL) {
    // Delete module variables.
    if (!isset($variable_patterns)) {
      $variable_patterns = [$module . '_%'];
    }
    $query = $this->getSelectQuery('variable', 'v')
      ->fields('v', ['name']);
    $or = new \DatabaseCondition('OR');
    foreach ($variable_patterns as $pattern) {
      $or->condition('name', $pattern, 'LIKE');
    }
    $query->condition($or);
    foreach ($query->execute() as $record) {
      variable_del($record->name);
    }

    // Delete the module itself.
    $this->getDeleteQuery('system')
      ->condition('name', $module)
      ->condition('type', 'module')
      ->execute();

    // Optionally drop schema tables.
    if ($tables) {
      $schema = $this->database->schema();
      foreach ($tables as $table) {
        $schema->dropTable($table);
      }
      cache_clear_all('schema', 'cache');
    }
  }

}
